// Klasse, die Ordnungs-Fragen realisiert
class OrdnungsFrage extends Frage {

	String[] antworten; // die zu ordnenden Dinge
	int[] geordneteIndizes; // Indizes in der korrekten Ordnungsreihenfolge

	public OrdnungsFrage(String text, int punkte, String[] antworten,
			int[] indizes) {
		super(text, punkte);
		this.antworten = antworten;
		this.geordneteIndizes = indizes;
	}

	@Override
	public void frageStellen() {
		super.frageStellen();
		for (int f = 0; f < this.antworten.length; f++) {
			IO.println("(" + f + "): " + this.antworten[f]);
		}
	}

	@Override
	public void frageBeantworten(Pruefling person) {
		int[] eingaben = new int[this.geordneteIndizes.length];
		boolean ok = true;
		for (int i = 0; i < this.geordneteIndizes.length; i++) {
			eingaben[i] = IO.readInt(i + 1 + ". : ");
			ok = ok && eingaben[i] == this.geordneteIndizes[i];
		}

		if (ok) {
			IO.println("Richtige Reihenfolge: " + this.punkte + " Punkte");
			person.neuePunkte(this.punkte);
		} else {
			IO.println("Falsche Reihenfolge: 0 Punkte!");
			IO.print("Richtige Reihenfolge ist: ");
			for (int i = 0; i < this.geordneteIndizes.length - 1; i++) {
				IO.print(this.geordneteIndizes[i] + ", ");
			}
			IO.println(this.geordneteIndizes[this.geordneteIndizes.length - 1]);
		}
	}
}

public class UE19Aufgabe4 {

	// Hauptprogramm
	public static void main(String[] args) {
		Pruefung klausur = new Pruefung();
		klausur.vorbereiten();
		klausur.durchfuehren();
		klausur.ergebnisseBekanntgeben();
	}

	Quiz pruefung;

	Pruefling[] studenten;

	UE19Aufgabe4() {
		this.pruefung = null;
		this.studenten = null;
	}

	void vorbereiten() {
		IO.println("Fragen eingeben");
		IO.println("----------------");
		String titel = IO.readString("Titel des Quizes: ");
		int anzahl = IO.readInt("Anzahl Fragen: ");
		this.pruefung = new Quiz(titel, anzahl);
		// Fragen eingeben
		for (int i = 0; i < anzahl; i++) {
			Frage f = this.frageErzeugen(i + 1);
			this.pruefung.neueFrage(f);
		}
	}

	public Frage frageErzeugen(int nummer) {
		// spaeter Factory-Pattern
		int typ = IO
				.readInt("Fragetyp: Wahr/Falsch (1), Multiple Choice (2), Ordnungsfrage (3)?");
		switch (typ) {
		case 1:
			return this.erzeugeWahrFalschFrage(nummer);
		case 2:
			return this.erzeugeMCFrage(nummer);
		case 3:
		default:
			return this.erzeugeOrdnungsFrage(nummer);
		}
	}

	private Frage erzeugeWahrFalschFrage(int nummer) {
		String text = IO.readString("Frage " + nummer + ": ");
		boolean wahr = IO.readChar("Wahr/falsch(w/f)?") == 'w';
		int punkte = IO.readInt("Erreichbare Punkte: ");
		return new WahrFalschFrage(text, punkte, wahr);
	}

	private Frage erzeugeMCFrage(int nummer) {
		String text = IO.readString("Frage " + nummer + ": ");
		int anzahl = IO.readInt("Anzahl an Antworten: ");
		String[] antworten = new String[anzahl];
		for (int i = 0; i < anzahl; i++) {
			antworten[i] = IO.readString("Antwort " + i + ": ");
		}
		int richtigIndex = IO.readInt("Index der richtigen Antwort: ");
		int punkte = IO.readInt("Erreichbare Punkte: ");
		return new MCFrage(text, punkte, antworten, richtigIndex);
	}

	private Frage erzeugeOrdnungsFrage(int nummer) {
		String text = IO.readString("Frage " + nummer + ": ");
		int anzahl = IO.readInt("Anzahl an Antworten: ");
		String[] antworten = new String[anzahl];
		for (int i = 0; i < anzahl; i++) {
			antworten[i] = IO.readString("Antwort " + i + ": ");
		}
		int[] indizes = new int[anzahl];
		for (int i = 0; i < anzahl; i++) {
			indizes[i] = IO.readInt(i + 1 + ". Index: ");
		}
		int punkte = IO.readInt("Erreichbare Punkte: ");
		return new OrdnungsFrage(text, punkte, antworten, indizes);
	}

	void durchfuehren() {
		IO.println("Pruefung");
		IO.println("--------");
		int anzahl = IO.readInt("Anzahl Prueflinge: ");
		this.studenten = new Pruefling[anzahl];
		// alle Prueflinge abfragen
		for (int i = 0; i < anzahl; i++) {
			IO.println("Pruefling " + (i + 1) + " ist an der Reihe");
			this.studenten[i] = new Pruefling(IO.readString("Name: "));
			Frage f = null;
			// alle Fragen der Pruefung stellen
			while ((f = this.pruefung.liefereNaechsteFrage()) != null) {
				f.frageStellen();
				f.frageBeantworten(this.studenten[i]);
			}
		}
	}

	void ergebnisseBekanntgeben() {
		this.ranglisteErstellen();
		IO.println("Pruefungsergebnisse");
		IO.println("-------------------");
		IO.println("Quiz: " + this.pruefung.getTitel());
		for (int i = 0; i < this.studenten.length; i++) {
			IO.println("Platz " + (i + 1) + ": " + this.studenten[i].getName()
					+ " mit " + this.studenten[i].getPunkte() + " Punkten");
		}
	}

	private void ranglisteErstellen() {
		// Bubblesort nach erreichten Punkten
		boolean veraendert = false;
		do {
			veraendert = false;
			for (int i = 0; i < this.studenten.length - 1; i++) {
				if (this.studenten[i].getPunkte() < this.studenten[i + 1]
						.getPunkte()) {
					Pruefling help = this.studenten[i];
					this.studenten[i] = this.studenten[i + 1];
					this.studenten[i + 1] = help;
					veraendert = true;
				}
			}
		} while (veraendert);
	}
}
